@@ -359,11 +359,6 @@ gsd_smartcard_manager_init (GsdSmartcardManager *manager)
                                        g_str_equal,
                                        (GDestroyNotify) g_free,
                                        (GDestroyNotify) g_object_unref);
-
-        if (!g_thread_supported ()) {
-                g_thread_init (NULL);
-        }
-
 }
 
 static void
@@ -396,6 +391,12 @@ gsd_smartcard_manager_error_quark (void)
         return error_quark;
 }
 
+GsdSmartcardManager *
+gsd_smartcard_manager_new_default (void)
+{
+  return gsd_smartcard_manager_new (NULL);
+}
+
 GsdSmartcardManager *
 gsd_smartcard_manager_new (const char *module_path)
 {
