@@ -66,6 +66,7 @@ struct GsdUpdatesManagerPrivate
         GDBusConnection         *connection;
         guint                    owner_id;
         GDBusNodeInfo           *introspection;
+        GPtrArray               *update_packages;
 };
 
 static void gsd_updates_manager_class_init (GsdUpdatesManagerClass *klass);
@@ -78,18 +79,38 @@ G_DEFINE_TYPE (GsdUpdatesManager, gsd_updates_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
+static void
+child_exit_cb (GPid pid, gint status, gpointer user_data)
+{
+        g_spawn_close_pid (pid);
+}
+
 static void
 clear_offline_updates_message (void)
 {
         gboolean ret;
         GError *error = NULL;
-        ret = g_spawn_command_line_async ("pkexec " LIBEXECDIR "/pk-clear-offline-update",
-                                          &error);
+        gchar *argv[3];
+        GPid pid;
+
+        argv[0] = BINDIR "/pkexec";
+        argv[1] = LIBEXECDIR "/pk-clear-offline-update";
+        argv[2] = NULL;
+        ret = g_spawn_async (NULL,
+                             argv,
+                             NULL,
+                             G_SPAWN_DO_NOT_REAP_CHILD,
+                             NULL,
+                             NULL,
+                             &pid,
+                             &error);
         if (!ret) {
                 g_warning ("Failure clearing offline update message: %s",
                            error->message);
                 g_error_free (error);
+                return;
         }
+        g_child_watch_add (pid, child_exit_cb, NULL);
 }
 
 static void
@@ -188,6 +209,7 @@ show_offline_updates_error (GsdUpdatesManager *manager)
                                   dialog);
         gtk_widget_show (dialog);
         clear_offline_updates_message ();
+        g_string_free (msg, TRUE);
 }
 
 static void
@@ -199,6 +221,7 @@ libnotify_action_cb (NotifyNotification *notification,
         GError *error = NULL;
         GsdUpdatesManager *manager = GSD_UPDATES_MANAGER (user_data);
 
+        notify_notification_close (notification, NULL);
         if (g_strcmp0 (action, "distro-upgrade-info") == 0) {
                 ret = g_spawn_command_line_async (DATADIR "/PackageKit/pk-upgrade-distro.sh",
                                                   &error);
@@ -245,6 +268,12 @@ out:
         return;
 }
 
+static void
+on_notification_closed (NotifyNotification *notification, gpointer data)
+{
+        g_object_unref (notification);
+}
+
 static void
 get_distro_upgrades_finished_cb (GObject *object,
                                  GAsyncResult *res,
@@ -331,6 +360,8 @@ get_distro_upgrades_finished_cb (GObject *object,
                                         _("More information"),
                                         libnotify_action_cb,
                                         manager, NULL);
+        g_signal_connect (notification, "closed",
+                          G_CALLBACK (on_notification_closed), NULL);
         ret = notify_notification_show (notification, &error);
         if (!ret) {
                 g_warning ("error: %s", error->message);
@@ -454,6 +485,8 @@ notify_critical_updates (GsdUpdatesManager *manager, GPtrArray *array)
         notify_notification_add_action (notification, "show-update-viewer",
                                         /* TRANSLATORS: button: open the update viewer to install updates*/
                                         _("Install updates"), libnotify_action_cb, manager, NULL);
+        g_signal_connect (notification, "closed",
+                          G_CALLBACK (on_notification_closed), NULL);
         ret = notify_notification_show (notification, &error);
         if (!ret) {
                 g_warning ("error: %s", error->message);
@@ -513,6 +546,8 @@ notify_normal_updates_maybe (GsdUpdatesManager *manager, GPtrArray *array)
         notify_notification_add_action (notification, "show-update-viewer",
                                         /* TRANSLATORS: button: open the update viewer to install updates*/
                                         _("Install updates"), libnotify_action_cb, manager, NULL);
+        g_signal_connect (notification, "closed",
+                          G_CALLBACK (on_notification_closed), NULL);
         ret = notify_notification_show (notification, &error);
         if (!ret) {
                 g_warning ("error: %s", error->message);
@@ -528,194 +563,6 @@ notify_normal_updates_maybe (GsdUpdatesManager *manager, GPtrArray *array)
         manager->priv->notification_updates = notification;
 }
 
-static gboolean
-update_check_on_battery (GsdUpdatesManager *manager)
-{
-        const gchar *message;
-        gboolean ret;
-        GError *error = NULL;
-        NotifyNotification *notification;
-
-        ret = g_settings_get_boolean (manager->priv->settings_gsd,
-                                      GSD_SETTINGS_UPDATE_BATTERY);
-        if (ret) {
-                g_debug ("okay to update due to policy");
-                return TRUE;
-        }
-
-        ret = gsd_updates_refresh_get_on_battery (manager->priv->refresh);
-        if (!ret) {
-                g_debug ("okay to update as on AC");
-                return TRUE;
-        }
-
-        /* do we do the notification? */
-        ret = g_settings_get_boolean (manager->priv->settings_gsd,
-                                      GSD_SETTINGS_NOTIFY_UPDATE_NOT_BATTERY);
-        if (!ret) {
-                g_debug ("ignoring due to GSettings");
-                return FALSE;
-        }
-
-        /* TRANSLATORS: policy says update, but we are on battery and so prompt */
-        message = _("Automatic updates are not being installed as the computer is running on battery power");
-        /* TRANSLATORS: informs user will not install by default */
-        notification = notify_notification_new (_("Updates not installed"),
-                                                message,
-                                                GSD_UPDATES_ICON_NORMAL);
-        notify_notification_set_app_name (notification, _("Software Updates"));
-        notify_notification_set_timeout (notification, 15000);
-        notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
-        notify_notification_add_action (notification, "update-all-packages",
-                                        /* TRANSLATORS: to hell with my battery life, just do it */
-                                        _("Install the updates anyway"), libnotify_action_cb, manager, NULL);
-        ret = notify_notification_show (notification, &error);
-        if (!ret) {
-                g_warning ("error: %s", error->message);
-                g_error_free (error);
-        }
-
-        return FALSE;
-}
-
-static const gchar *
-restart_enum_to_localised_text (PkRestartEnum restart)
-{
-        const gchar *text = NULL;
-        switch (restart) {
-        case PK_RESTART_ENUM_NONE:
-                text = _("No restart is required.");
-                break;
-        case PK_RESTART_ENUM_SYSTEM:
-                text = _("A restart is required.");
-                break;
-        case PK_RESTART_ENUM_SESSION:
-                text = _("You need to log out and log back in.");
-                break;
-        case PK_RESTART_ENUM_APPLICATION:
-                text = _("You need to restart the application.");
-                break;
-        case PK_RESTART_ENUM_SECURITY_SESSION:
-                text = _("You need to log out and log back in to remain secure.");
-                break;
-        case PK_RESTART_ENUM_SECURITY_SYSTEM:
-                text = _("A restart is required to remain secure.");
-                break;
-        default:
-                g_warning ("restart unrecognised: %i", restart);
-        }
-        return text;
-}
-
-static void
-notify_update_finished (GsdUpdatesManager *manager, PkResults *results)
-{
-        const gchar *message;
-        gboolean ret;
-        gchar *package_id = NULL;
-        gchar **split;
-        gchar *summary = NULL;
-        GError *error = NULL;
-        GPtrArray *array;
-        GString *message_text = NULL;
-        guint i;
-        guint skipped_number = 0;
-        NotifyNotification *notification;
-        PkInfoEnum info;
-        PkPackage *item;
-        PkRestartEnum restart;
-
-        /* check we got some packages */
-        array = pk_results_get_package_array (results);
-        g_debug ("length=%i", array->len);
-        if (array->len == 0) {
-                g_debug ("no updates");
-                goto out;
-        }
-
-        message_text = g_string_new ("");
-
-        /* find any we skipped */
-        for (i=0; i<array->len; i++) {
-                item = g_ptr_array_index (array, i);
-                g_object_get (item,
-                              "info", &info,
-                              "package-id", &package_id,
-                              "summary", &summary,
-                              NULL);
-
-                split = pk_package_id_split (package_id);
-                g_debug ("%s, %s, %s", pk_info_enum_to_string (info),
-                         split[PK_PACKAGE_ID_NAME], summary);
-                if (info == PK_INFO_ENUM_BLOCKED) {
-                        skipped_number++;
-                        g_string_append_printf (message_text, "<b>%s</b> - %s\n",
-                                                split[PK_PACKAGE_ID_NAME], summary);
-                }
-                g_free (package_id);
-                g_free (summary);
-                g_strfreev (split);
-        }
-
-        /* notify the user if there were skipped entries */
-        if (skipped_number > 0) {
-                /* TRANSLATORS: we did the update, but some updates were skipped and not applied */
-                message = ngettext ("One package was skipped:",
-                                    "Some packages were skipped:", skipped_number);
-                g_string_prepend (message_text, message);
-                g_string_append_c (message_text, '\n');
-        }
-
-        /* add a message that we need to restart */
-        restart = pk_results_get_require_restart_worst (results);
-        if (restart != PK_RESTART_ENUM_NONE) {
-                message = restart_enum_to_localised_text (restart);
-
-                /* add a gap if we are putting both */
-                if (skipped_number > 0)
-                        g_string_append (message_text, "\n");
-
-                g_string_append (message_text, message);
-                g_string_append_c (message_text, '\n');
-        }
-
-        /* trim off extra newlines */
-        if (message_text->len != 0)
-                g_string_set_size (message_text, message_text->len-1);
-
-        /* do we do the notification? */
-        ret = g_settings_get_boolean (manager->priv->settings_gsd,
-                                      GSD_SETTINGS_NOTIFY_UPDATE_COMPLETE);
-        if (!ret) {
-                g_debug ("ignoring due to GSettings");
-                goto out;
-        }
-
-        /* TRANSLATORS: title: system update completed all okay */
-        notification = notify_notification_new (_("The system update has completed"),
-                                                 message_text->str,
-                                                 GSD_UPDATES_ICON_NORMAL);
-        notify_notification_set_app_name (notification, _("Software Updates"));
-        notify_notification_set_timeout (notification, 15000);
-        notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
-        if (restart == PK_RESTART_ENUM_SYSTEM) {
-                notify_notification_add_action (notification, "restart",
-                                                /* TRANSLATORS: restart computer as system packages need update */
-                                                _("Restart computer now"),
-                                                libnotify_action_cb,
-                                                manager, NULL);
-        }
-        ret = notify_notification_show (notification, &error);
-        if (!ret) {
-                g_warning ("error: %s", error->message);
-                g_error_free (error);
-        }
-out:
-        if (message_text != NULL)
-                g_string_free (message_text, TRUE);
-        g_ptr_array_unref (array);
-}
-
 static void
 update_packages_finished_cb (PkTask *task,
                              GAsyncResult *res,
@@ -744,7 +591,6 @@ update_packages_finished_cb (PkTask *task,
         }
 
         /* notify */
-        notify_update_finished (manager, results);
         manager->priv->number_updates_critical_last_shown = 0;
 out:
         if (error_code != NULL)
@@ -790,6 +636,8 @@ notify_failed_get_updates_maybe (GsdUpdatesManager *manager)
                                         button,
                                         libnotify_action_cb,
                                         manager, NULL);
+        g_signal_connect (notification, "closed",
+                          G_CALLBACK (on_notification_closed), NULL);
         ret = notify_notification_show (notification, &error);
         if (!ret) {
                 g_warning ("failed to show notification: %s",
@@ -801,6 +649,31 @@ out:
         manager->priv->failed_get_updates_count = 0;
 }
 
+static void
+check_updates_for_importance (GsdUpdatesManager *manager)
+{
+        guint i;
+        PkPackage *pkg;
+        GPtrArray *important_array;
+
+        /* check each package */
+        important_array = g_ptr_array_new ();
+        for (i = 0; i < manager->priv->update_packages->len; i++) {
+                pkg = g_ptr_array_index (manager->priv->update_packages, i);
+                if (pk_package_get_info (pkg) == PK_INFO_ENUM_SECURITY ||
+                    pk_package_get_info (pkg) == PK_INFO_ENUM_IMPORTANT)
+                        g_ptr_array_add (important_array, pkg);
+        }
+        if (important_array->len > 0) {
+                notify_critical_updates (manager,
+                                         important_array);
+        } else {
+                notify_normal_updates_maybe (manager,
+                                             manager->priv->update_packages);
+        }
+        g_ptr_array_unref (important_array);
+}
+
 static void
 package_download_finished_cb (GObject *object,
                               GAsyncResult *res,
@@ -831,6 +704,9 @@ package_download_finished_cb (GObject *object,
                 goto out;
         }
 
+        /* check to see if should notify */
+        check_updates_for_importance (manager);
+
         /* we succeeded, so allow the shell to query us */
         manager->priv->pending_updates = TRUE;
         emit_changed (manager);
@@ -842,20 +718,28 @@ out:
 }
 
 static void
-auto_download_updates (GsdUpdatesManager *manager,
-                       GPtrArray *array)
+auto_download_updates (GsdUpdatesManager *manager)
 {
         gchar **package_ids;
         guint i;
         PkPackage *pkg;
 
         /* download each package */
-        package_ids = g_new0 (gchar *, array->len + 1);
-        for (i=0; i<array->len; i++) {
-                pkg = g_ptr_array_index (array, i);
+        package_ids = g_new0 (gchar *, manager->priv->update_packages->len + 1);
+        for (i = 0; i < manager->priv->update_packages->len; i++) {
+                pkg = g_ptr_array_index (manager->priv->update_packages, i);
                 package_ids[i] = g_strdup (pk_package_get_id (pkg));
         }
 
+#if PK_CHECK_VERSION(0,1,8)
+        /* we've set only-download in PkTask */
+        pk_task_update_packages_async (manager->priv->task,
+                                       package_ids,
+                                       manager->priv->cancellable,
+                                       NULL, NULL,
+                                       (GAsyncReadyCallback) package_download_finished_cb,
+                                       manager);
+#else
         /* download them all */
         pk_client_download_packages_async (PK_CLIENT(manager->priv->task),
                                            package_ids,
@@ -864,6 +748,7 @@ auto_download_updates (GsdUpdatesManager *manager,
                                            NULL, NULL,
                                            (GAsyncReadyCallback) package_download_finished_cb,
                                            manager);
+#endif
         g_strfreev (package_ids);
 }
 
@@ -875,13 +760,7 @@ get_updates_finished_cb (GObject *object,
         PkClient *client = PK_CLIENT(object);
         PkResults *results;
         GError *error = NULL;
-        PkPackage *item;
-        guint i;
         gboolean ret;
-        GsdUpdateType update;
-        GPtrArray *security_array = NULL;
-        gchar **package_ids;
-        GPtrArray *array = NULL;
         PkError *error_code = NULL;
 
         /* get the results */
@@ -907,103 +786,30 @@ get_updates_finished_cb (GObject *object,
         /* we succeeded, so clear the count */
         manager->priv->failed_get_updates_count = 0;
 
-        /* get data */
-        array = pk_results_get_package_array (results);
+        /* so we can download or check for important & security updates */
+        if (manager->priv->update_packages != NULL)
+                g_ptr_array_unref (manager->priv->update_packages);
+        manager->priv->update_packages = pk_results_get_package_array (results);
 
         /* we have no updates */
-        if (array->len == 0) {
+        if (manager->priv->update_packages->len == 0) {
                 g_debug ("no updates");
                 goto out;
         }
 
-        /* we have updates to process */
-        security_array = g_ptr_array_new_with_free_func (g_free);
-
-        /* find the security updates first */
-        for (i=0; i<array->len; i++) {
-                item = g_ptr_array_index (array, i);
-                if (pk_package_get_info (item) != PK_INFO_ENUM_SECURITY)
-                        continue;
-                g_ptr_array_add (security_array, g_strdup (pk_package_get_id (item)));
-        }
-
-        /* do we do the automatic updates? */
-        update = g_settings_get_enum (manager->priv->settings_gsd,
-                                      GSD_SETTINGS_AUTO_UPDATE_TYPE);
-
-        /* is policy none? */
-        if (update == GSD_UPDATE_TYPE_NONE) {
-                g_debug ("not updating as policy NONE");
-                /* do we warn the user? */
-                if (security_array->len > 0)
-                        notify_critical_updates (manager, security_array);
-                else
-                        notify_normal_updates_maybe (manager, array);
-
-                /* should we auto-download other updates? */
-                ret = g_settings_get_boolean (manager->priv->settings_gsd,
-                                              GSD_SETTINGS_AUTO_DOWNLOAD_UPDATES);
-                if (ret)
-                        auto_download_updates (manager, array);
+        /* should we auto-download the updates? */
+        ret = g_settings_get_boolean (manager->priv->settings_gsd,
+                                      GSD_SETTINGS_AUTO_DOWNLOAD_UPDATES);
+        if (ret) {
+                auto_download_updates (manager);
                 goto out;
         }
 
-        /* are we on battery and configured to skip the action */
-        ret = update_check_on_battery (manager);
-        if (!ret &&
-            ((update == GSD_UPDATE_TYPE_SECURITY && security_array->len > 0) ||
-              update == GSD_UPDATE_TYPE_ALL)) {
-                g_debug ("on battery so not doing update");
-                if (security_array->len > 0)
-                        notify_critical_updates (manager, security_array);
-                goto out;
-        }
-
-        /* just do security updates */
-        if (update == GSD_UPDATE_TYPE_SECURITY) {
-                if (security_array->len == 0) {
-                        g_debug ("policy security, but none available");
-                        notify_normal_updates_maybe (manager, array);
-                        goto out;
-                }
-
-                /* even if this is TRUE, we're doing something about it */
-                manager->priv->pending_updates = FALSE;
-
-                /* convert */
-                package_ids = pk_ptr_array_to_strv (security_array);
-                pk_task_update_packages_async (manager->priv->task, package_ids,
-                                               manager->priv->cancellable,
-                                               NULL, NULL,
-                                               (GAsyncReadyCallback) update_packages_finished_cb, manager);
-                g_strfreev (package_ids);
-                goto out;
-        }
-
-        /* just do everything */
-        if (update == GSD_UPDATE_TYPE_ALL) {
-
-                /* even if this is TRUE, we're doing something about it */
-                manager->priv->pending_updates = FALSE;
-
-                g_debug ("we should do the update automatically!");
-                pk_task_update_system_async (manager->priv->task,
-                                             manager->priv->cancellable,
-                                             NULL, NULL,
-                                             (GAsyncReadyCallback) update_packages_finished_cb,
-                                             manager);
-                goto out;
-        }
-
-        /* shouldn't happen */
-        g_warning ("unknown update mode");
+        /* just check to see if should notify */
+        check_updates_for_importance (manager);
 out:
         if (error_code != NULL)
                 g_object_unref (error_code);
-        if (security_array != NULL)
-                g_ptr_array_unref (security_array);
-        if (array != NULL)
-                g_ptr_array_unref (array);
         if (results != NULL)
                 g_object_unref (results);
 }
@@ -1171,7 +977,7 @@ session_inhibit (GsdUpdatesManager *manager)
 {
         const gchar *reason;
         GError *error = NULL;
-        GVariant *retval;
+        GVariant *retval = NULL;
 
         /* state invalid somehow */
         if (manager->priv->inhibit_cookie != 0) {
@@ -1203,14 +1009,15 @@ session_inhibit (GsdUpdatesManager *manager)
         g_variant_get (retval, "(u)",
                        &manager->priv->inhibit_cookie);
 out:
-        return;
+        if (retval != NULL)
+                g_variant_unref (retval);
 }
 
 static void
 session_uninhibit (GsdUpdatesManager *manager)
 {
         GError *error = NULL;
-        GVariant *retval;
+        GVariant *retval = NULL;
 
         /* state invalid somehow */
         if (manager->priv->inhibit_cookie == 0) {
@@ -1233,7 +1040,8 @@ session_uninhibit (GsdUpdatesManager *manager)
         }
 out:
         manager->priv->inhibit_cookie = 0;
-        return;
+        if (retval != NULL)
+                g_variant_unref (retval);
 }
 
 static void
@@ -1527,6 +1335,8 @@ check_offline_update_cb (gpointer user_data)
         notify_notification_add_action (notification, "clear-offline-updates",
                                         /* TRANSLATORS: button: clear notification */
                                         _("OK"), libnotify_action_cb, manager, NULL);
+        g_signal_connect (notification, "closed",
+                          G_CALLBACK (on_notification_closed), NULL);
         ret = notify_notification_show (notification, &error);
         if (!ret) {
                 g_warning ("error: %s", error->message);
@@ -1666,54 +1476,18 @@ gsd_updates_manager_stop (GsdUpdatesManager *manager)
 {
         g_debug ("Stopping updates manager");
 
-        if (manager->priv->settings_proxy != NULL) {
-                g_object_unref (manager->priv->settings_proxy);
-                manager->priv->settings_proxy = NULL;
-        }
-        if (manager->priv->settings_http != NULL) {
-                g_object_unref (manager->priv->settings_http);
-                manager->priv->settings_http = NULL;
-        }
-        if (manager->priv->settings_ftp != NULL) {
-                g_object_unref (manager->priv->settings_ftp);
-                manager->priv->settings_ftp = NULL;
-        }
-        if (manager->priv->settings_gsd != NULL) {
-                g_object_unref (manager->priv->settings_gsd);
-                manager->priv->settings_gsd = NULL;
-        }
-        if (manager->priv->control != NULL) {
-                g_object_unref (manager->priv->control);
-                manager->priv->control = NULL;
-        }
-        if (manager->priv->task != NULL) {
-                g_object_unref (manager->priv->task);
-                manager->priv->task = NULL;
-        }
-        if (manager->priv->refresh != NULL) {
-                g_object_unref (manager->priv->refresh);
-                manager->priv->refresh = NULL;
-        }
-        if (manager->priv->firmware != NULL) {
-                g_object_unref (manager->priv->firmware);
-                manager->priv->firmware = NULL;
-        }
-        if (manager->priv->proxy_session != NULL) {
-                g_object_unref (manager->priv->proxy_session);
-                manager->priv->proxy_session = NULL;
-        }
-        if (manager->priv->volume_monitor != NULL) {
-                g_object_unref (manager->priv->volume_monitor);
-                manager->priv->volume_monitor = NULL;
-        }
-        if (manager->priv->cancellable != NULL) {
-                g_object_unref (manager->priv->cancellable);
-                manager->priv->cancellable = NULL;
-        }
-        if (manager->priv->introspection != NULL) {
-                g_dbus_node_info_unref (manager->priv->introspection);
-                manager->priv->introspection = NULL;
-        }
+        g_clear_object (&manager->priv->settings_proxy);
+        g_clear_object (&manager->priv->settings_http);
+        g_clear_object (&manager->priv->settings_ftp);
+        g_clear_object (&manager->priv->settings_gsd);
+        g_clear_object (&manager->priv->control);
+        g_clear_object (&manager->priv->task);
+        g_clear_object (&manager->priv->refresh);
+        g_clear_object (&manager->priv->firmware);
+        g_clear_object (&manager->priv->proxy_session);
+        g_clear_object (&manager->priv->volume_monitor);
+        g_clear_object (&manager->priv->cancellable);
+        g_clear_pointer (&manager->priv->introspection, g_dbus_node_info_unref);
         if (manager->priv->update_viewer_watcher_id != 0) {
                 g_bus_unwatch_name (manager->priv->update_viewer_watcher_id);
                 manager->priv->update_viewer_watcher_id = 0;
@@ -1726,10 +1500,11 @@ gsd_updates_manager_stop (GsdUpdatesManager *manager)
                 g_source_remove (manager->priv->offline_update_id);
                 manager->priv->offline_update_id = 0;
         }
-        if (manager->priv->offline_update_error != NULL) {
-                g_object_unref (manager->priv->offline_update_error);
-                manager->priv->offline_update_error = 0;
+        if (manager->priv->update_packages != NULL) {
+                g_ptr_array_unref (manager->priv->update_packages);
+                manager->priv->update_packages = NULL;
         }
+        g_clear_object (&manager->priv->offline_update_error);
 }
 
 static GObject *
