@@ -233,10 +233,48 @@ set_area (GsdWacomDevice  *device,
         g_variant_unref (value);
 }
 
+/* Returns the rotation to apply a device relative to the current rotation of the output */
+static GsdWacomRotation
+get_relative_rotation (GsdWacomRotation device_rotation,
+                       GsdWacomRotation output_rotation)
+{
+	GsdWacomRotation rotations[] = { GSD_WACOM_ROTATION_HALF,
+	                                 GSD_WACOM_ROTATION_CW,
+	                                 GSD_WACOM_ROTATION_NONE,
+	                                 GSD_WACOM_ROTATION_CCW };
+	guint i;
+
+	if (device_rotation == output_rotation)
+		return GSD_WACOM_ROTATION_NONE;
+
+	if (output_rotation == GSD_WACOM_ROTATION_NONE)
+		return device_rotation;
+
+	for (i = 0; i < G_N_ELEMENTS (rotations); i++){
+		if (device_rotation == rotations[i])
+			break;
+	}
+
+	if (output_rotation == GSD_WACOM_ROTATION_HALF)
+		return rotations[(i + G_N_ELEMENTS (rotations) - 2) % G_N_ELEMENTS (rotations)];
+
+	if (output_rotation == GSD_WACOM_ROTATION_CW)
+		return rotations[(i + G_N_ELEMENTS (rotations) - 1) % G_N_ELEMENTS (rotations)];
+
+	if (output_rotation == GSD_WACOM_ROTATION_CCW)
+		return rotations[(i + 1) % G_N_ELEMENTS (rotations)];
+
+	/* fallback */
+	return GSD_WACOM_ROTATION_NONE;
+}
+
 static void
 set_display (GsdWacomDevice  *device,
              GVariant        *value)
 {
+        GsdWacomRotation  device_rotation;
+	GsdWacomRotation  output_rotation;
+	GSettings        *settings;
         float matrix[NUM_ELEMS_MATRIX];
         PropertyHelper property = {
                 .name   = "Coordinate Transformation Matrix",
@@ -251,6 +289,14 @@ set_display (GsdWacomDevice  *device,
         g_debug ("Applying matrix to device...");
         wacom_set_property (device, &property);
 
+        /* Compute rotation to apply relative to the output */
+	settings = gsd_wacom_device_get_settings (device);
+	device_rotation = g_settings_get_enum (settings, KEY_ROTATION);
+	output_rotation = gsd_wacom_device_get_display_rotation (device);
+
+        /* Apply display rotation to device */
+        set_rotation (device, get_relative_rotation (device_rotation, output_rotation));
+
         g_variant_unref (value);
 }
 
@@ -689,9 +735,13 @@ set_wacom_settings (GsdWacomManager *manager,
 		set_tpcbutton (device, g_settings_get_boolean (settings, KEY_TPCBUTTON));
 
 	set_absolute (device, g_settings_get_boolean (settings, KEY_IS_ABSOLUTE));
-	if (!gsd_wacom_device_is_screen_tablet (device))
-		set_keep_aspect (device, g_settings_get_boolean (settings, KEY_KEEP_ASPECT));
-	set_area (device, g_settings_get_value (settings, KEY_AREA));
+
+	/* Ignore touch devices as they do not share the same range of values for area */
+	if (type != WACOM_TYPE_TOUCH) {
+		if (gsd_wacom_device_is_screen_tablet (device) == FALSE)
+			set_keep_aspect (device, g_settings_get_boolean (settings, KEY_KEEP_ASPECT));
+		set_area (device, g_settings_get_value (settings, KEY_AREA));
+	}
 	set_display (device, g_settings_get_value (settings, KEY_DISPLAY));
 
         /* only pen and eraser have pressure threshold and curve settings */
@@ -711,7 +761,8 @@ wacom_settings_changed (GSettings      *settings,
 	type = gsd_wacom_device_get_device_type (device);
 
 	if (g_str_equal (key, KEY_ROTATION)) {
-		set_rotation (device, g_settings_get_enum (settings, key));
+	        if (type != WACOM_TYPE_PAD)
+		        set_rotation (device, g_settings_get_enum (settings, key));
 	} else if (g_str_equal (key, KEY_TOUCH)) {
 		set_touch (device, g_settings_get_boolean (settings, key));
 	} else if (g_str_equal (key, KEY_TPCBUTTON)) {
@@ -723,7 +774,8 @@ wacom_settings_changed (GSettings      *settings,
 			set_absolute (device, g_settings_get_boolean (settings, key));
 	} else if (g_str_equal (key, KEY_AREA)) {
 		if (type != WACOM_TYPE_CURSOR &&
-		    type != WACOM_TYPE_PAD)
+		    type != WACOM_TYPE_PAD &&
+		    type != WACOM_TYPE_TOUCH)
 			set_area (device, g_settings_get_value (settings, key));
 	} else if (g_str_equal (key, KEY_DISPLAY)) {
 		if (type != WACOM_TYPE_CURSOR &&
@@ -732,6 +784,7 @@ wacom_settings_changed (GSettings      *settings,
 	} else if (g_str_equal (key, KEY_KEEP_ASPECT)) {
 		if (type != WACOM_TYPE_CURSOR &&
 		    type != WACOM_TYPE_PAD &&
+		    type != WACOM_TYPE_TOUCH &&
 		    !gsd_wacom_device_is_screen_tablet (device))
 			set_keep_aspect (device, g_settings_get_boolean (settings, key));
 	} else {
@@ -970,24 +1023,35 @@ generate_key (GsdWacomTabletButton *wbutton,
 	/* Couldn't find it in the current group? Look in group 0 */
 	if (keycode == 0) {
 		for (i = 0; i < n_keys; i++) {
-			if (keys[i].level > 0)
+			if (keys[i].group > 0)
 				continue;
 			keycode = keys[i].keycode;
 		}
 	}
 	g_free (keys);
 
-	g_debug ("Emitting '%s' (keyval: %d, keycode: %d mods: 0x%x)",
-		 str, keyval, keycode, mods);
-	g_free (str);
+	if (keycode == 0) {
+		g_warning ("Not emitting '%s' (keyval: %d, keycode: %d mods: 0x%x), invalid keycode",
+			   str, keyval, keycode, mods);
+		g_free (str);
+		return;
+	} else {
+		g_debug ("Emitting '%s' (keyval: %d, keycode: %d mods: 0x%x)",
+			 str, keyval, keycode, mods);
+	}
 
 	/* And send out the keys! */
+	gdk_error_trap_push ();
 	if (is_press)
 		send_modifiers (display, mods, TRUE);
 	XTestFakeKeyEvent (display, keycode,
 			   is_press ? True : False, 0);
 	if (is_press == FALSE)
 		send_modifiers (display, mods, FALSE);
+	if (gdk_error_trap_pop ())
+		g_warning ("Failed to generate fake key event '%s'", str);
+
+	g_free (str);
 }
 
 static GdkFilterReturn
@@ -1139,6 +1203,12 @@ on_screen_changed_cb (GnomeRRScreen *rr_screen,
 			continue;
 
 		settings = gsd_wacom_device_get_settings (device);
+		/* Ignore touch devices as they do not share the same range of values for area */
+		if (type != WACOM_TYPE_TOUCH) {
+			if (gsd_wacom_device_is_screen_tablet (device) == FALSE)
+				set_keep_aspect (device, g_settings_get_boolean (settings, KEY_KEEP_ASPECT));
+			set_area (device, g_settings_get_value (settings, KEY_AREA));
+		}
 		set_display (device, g_settings_get_value (settings, KEY_DISPLAY));
 	}
 	g_list_free (devices);
